`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    09:12:10 04/10/2013 
// Design Name: 
// Module Name:    MemoryInterface 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module MemoryInterface(
	//	Data connections
	input		wire	[15:0]	DataIn,			//	input data bus
	output	wire	[15:0]	DataOut,			//	output data bus
	
	//	Control connections
	input		wire				clk_in,			//	input clock - assume 50MHz
	input		wire				write_en,		//	enable writes TO memory (Acquisition from UUT)
	input		wire				write_clk,		//	write data valid at rising edge
	input		wire				read_en,			//	enable reads FROM memory
	input		wire				read_req,		//	request signal TO output FIFO to read data TO PC
	input		wire				read_clk,		//	requsted data should be valid on rising edge (ti_clk)
	input		wire				reset,			//	reset signal
	
	//	Status Connections
	output	wire				mem_full,		//	Signals that we've filled the memory (We've reached the last row address)
	output	wire				reading,			//	Indicates that data is being read in FROM DUT
	output	wire				writing,			//	Indicates that data is being written TO PC
	output	wire				fault_ifull,	//	Input FIFO is full during acquisition from UUT
	output	wire				fault_iempty,	//	Input FIFO is empty during acquisition from UUT
	output	wire				fault_ofull,	//	Output FIFO is full during data transfer to PC
	output	wire				fault_oempty,	//	Output FIFO is empty during data transfer to PC
	
	//	Hard connections to the SDRAM
	output	wire				sdram_clk_out,	//	The clcok that feeds the SDRAM
	output	wire				sdram_cke,
	output	wire				sdram_cs_n,
	output	wire				sdram_we_n,
	output	wire				sdram_cas_n,
	output	wire				sdram_ras_n,
	output	wire				sdram_ldqm,
	output	wire				sdram_udqm,
	output	wire [1:0]		sdram_ba,
	output	wire [12:0]		sdram_a,
	inout		wire [15:0]		sdram_d
   );
	 
//	 wire				gnd_bit;
//	 assign			gnd_bit = 1'b0;
	 
//--------------------------------------------------------------------------------
	//	Set these to constants
	assign	sdram_cke = 1'b1;
	assign	sdram_ldqm = 1'b0;
	assign	sdram_udqm = 1'b0;


//--------------------------------------------------------------------------------
	//	Create the 133MHz clock required for the SDRAM - Assume clock has already been buffered somewhere...
	wire		w_sdram_clk;
	assign	w_sdram_clk = clk_in;
	
		
//---------------------------------------------------------------------------------
	//  Instatiate the driver for the SDRAM clock -
		//	This is for when the clock is supplied by the FPGA fabric
		//	rather than by the PLL directly
	my_ofd SDRAM_CLK_Driver(
		.sdram_clk_out(sdram_clk_out),		//	Source to the SDRAM hardware
		.sdram_clk(w_sdram_clk)					//	133MHz clock
		);
		
		
//---------------------------------------------------------------------------------
	//	Synchronize the read and write enables 
	wire		w_sdram_rden;			//	Signal to send data TO PC
	wire		w_sdram_wren;			//	Signal to read data FROM UUT
	wire		w_sdram_reset;			//	Reset signal
	reg		r_sdram_rden;			//	SDRAM read enable to transfer negotiator (Read data to PC)
	reg		r_sdram_wren;			//	SDRAM write enable to transfer negotiator (Acquire data from UUT)
	reg		r_sdram_reset;			//	SDRAM/FIFO reset
	
	assign 	w_sdram_reset = r_sdram_reset;
	assign	w_sdram_wren = r_sdram_wren;
	assign	w_sdram_rden = r_sdram_rden;	
	
	
	// These signals come in on TI_CLK from the host interface.  We need
	// to make sure to resynchronize them to our state machine clock or
	// things strange things can happen (like hopping to unexpected states).
	always @(negedge w_sdram_clk) begin
		r_sdram_rden = read_en;
		r_sdram_wren = write_en;
		r_sdram_reset = reset;
	end

	 //	This basically just echoes our status back out
	 assign			writing = w_sdram_rden & read_req;		//	From the memory point of view, we are writing data to the PC
	 
	
//---------------------------------------------------------------------------------
	//	Instantiate the transfer negotiator - negotiates reads and writes to the SDRAM
	wire				w_cmd_ack;					//	Command acknowledge FROM SDRAM controller
	wire				w_cmd_done;					//	Command done signal FROM SDRAM controller
	wire				w_cmd_pagewrite;			//	page write command TO SDRAM controller
	wire				w_cmd_pageread;			//	page read command TO SDRAM controller
	wire	[10:0]	w_input_fifo_status;		//	number of bytes in read (input) FIFO
	wire	[10:0]	w_output_fifo_status;	//	number of bytes in write (output) FIFO
	wire	[14:0]	w_row_addr;					//	SDRAM row address for read/write
	

	//	This transfer negotiator expects all reads and writes to be a contiguos block
		//	By default, this controller starts at row address 0x0000, and keeps incrementing
		//	the row address by 1 regardless of what the command is, that is, a write command 
		//	will go to an address, and a read command will go to the following address.
		//	For random access workings, this would need to be modified to add a address input
		//	for the logic analyzer, due to the way we are collecting data, we have instituted
		//	a "memry full" output that lets us know when we've reached the last address
		//	so we can stop a read/write operation, and not have the row address roll over.
	Transfer_Negotiate	xfr_neg(
		.sdram_clk(w_sdram_clk),						//	Input Clock (SDRAM clock)
		.reset(w_sdram_reset),							//	reset input
		.sdram_wren(w_sdram_wren),						//	Write request FROM PC
		.sdram_rden(w_sdram_rden),						//	Read request FROM PC
		.cmd_ack(w_cmd_ack),								//	Command acknowledge FROM memory controller
		.cmd_done(w_cmd_done),							//	Command completion signal FROM memory controller
		.read_fifo_status(w_input_fifo_status),	//	Number of bytes in read fifo (are there enough to do a page write to SDRAM?)
		.write_fifo_status(w_output_fifo_status),	//	Number of bytes in write fifo (are there enough free to do a page read from SDRAM?)
		.cmd_pagewrite(w_cmd_pagewrite),				//	Page Write command TO memory controller
		.cmd_pageread(w_cmd_pageread),				//	Page read command TO memory controller
		.rowaddr(w_row_addr),							//	Row address for read/write
		.mem_full(mem_full)								//	Memory full indicator 
		);

//------------------------------------------------------------------------------------
	//	Instantiate the SDRAM controller
	wire	[15:0]		w_din_from_fifo;			//	connection to input FIFO
	wire	[15:0]		w_dout_to_fifo;			//	connection to output FIFO
	wire					w_output_fifo_write;		//	Write enable to OUTPUT FIFO
	wire					w_input_fifo_read;		//	Read request to INPUT FIFO
	
	//	This controller works on a page (512 words) at a time
		//	That is, every transfer (read or write) is 1024 bytes
	sdramctrl	/*#(
			.REFRESH_CYCLE(11'd1000)
			)*/
		SDRAM_ctrl(
			.clk(~w_sdram_clk),								//	133MHz clock (inverted)
			.clk_read(~w_sdram_clk),						//	133MHz clock (inverted)
			.reset(w_sdram_reset),							//	Controller reset input
			.cmd_pagewrite(w_cmd_pagewrite),				//	Page write command FROM transfer negotiator
			.cmd_pageread(w_cmd_pageread),				//	Page read command FROM rtansfer negotiator
			.cmd_ack(w_cmd_ack),								//	command acknowledge TO transfer negotiator
			.cmd_done(w_cmd_done),							//	Command completion signal TO transfer negotiator
			.rowaddr_in(w_row_addr),						//	Row address for read/write
			.fifo_din(w_din_from_fifo),					//	Data input FROM input FIFO
			.fifo_dout(w_dout_to_fifo),					//	Data output TO output FIFO
			.fifo_write(w_output_fifo_write),						//	Write enable TO OUTPUT FIFO
			.fifo_read(w_input_fifo_read),						//	Read request TO INPUT FIFO
			.sdram_cmd({sdram_cs_n, sdram_ras_n, sdram_cas_n, sdram_we_n}),		// cs_n, ras_n, cas_n, we_n}
			.sdram_ba(sdram_ba),								//	Bank Address TO the SDRAM
			.sdram_a(sdram_a),								//	Row address TO the SDRAM
			.sdram_d(sdram_d)									//	Data bus TO the SDRAM
			);


//-------------------------------------------------------------------------------------
	//	Instantiate input FIFO (data FROM outside world)
	wire					w_input_fifo_empty;
	wire					w_input_fifo_full;
//	wire					w_input_fifo_write;
	
	//	When the SDRAM gets filled, this FIFO will continue to fill until it is full
		//	unless we do something preventative...
	assign reading = (w_sdram_wren & ~mem_full);		//  From the memory's point of view, we are reading from the outside world

	fifo16w16r_2048 input_fifo (
		.rst(w_sdram_reset),							//	Resetting SDRAM controller also resets FIFO
		.rd_data_count(w_input_fifo_status),	//	How many bytes available to read?
		.wr_data_count(),								//	How many bytes written?
		.empty(w_input_fifo_empty),				//	FIFO empty
		.full(w_input_fifo_full),					//	FIFO full
		.wr_clk(write_clk),							//	Write clock from next higher module
		.wr_en(reading),								//	Write enable input TO the FIFO
		.din(DataIn),									//	Data Input from next higher module TO FIFO
		.rd_clk(~w_sdram_clk),						//	Need to get data on falling edge for timing
		.rd_en(w_input_fifo_read),					//	read request TO FIFO
		.dout(w_din_from_fifo)						//	Data FROM FIFO
		);

		
//-------------------------------------------------------------------------------------
	//	Instantiate output FIFO (data TO PC)
	wire					w_output_fifo_empty;
	wire					w_output_fifo_full;
	
	fifo16w16r_2048 output_fifo (
		.rst(w_sdram_reset),							//	Resetting SDRAM controller also resets FIFO
		.rd_data_count(),								//	How many bytes available to read?
		.wr_data_count(w_output_fifo_status),	//	How many bytes written?
		.empty(w_output_fifo_empty),				//	FIFO empty
		.full(w_output_fifo_full),					//	FIFO full
		.wr_clk(~w_sdram_clk),						//	Write clock from SDRAM
		.wr_en(w_output_fifo_write),				//	Write enable input TO the FIFO
		.din(w_dout_to_fifo),						//	Data Input FROM sdram controller
		.rd_clk(read_clk),							//	Need to get data on falling edge for timing
		.rd_en(read_req),								//	Read request TO FIFO (from external entity)
		.dout(DataOut)									//	Data out to next higher module
		);
		
//-------------------------------------------------------------------------------------
	//	Instantiate Error Detection
	FaultDetect	ouch(
			.sdram_clk(w_sdram_clk),					//	133MHz clock to SDRAM
			.ti_clk(read_clk),							//	48MHz clock from USB
			.reset(w_sdram_reset),						//	Memory reset
			.c0_fifo_write(w_output_fifo_write),	//	OUTPUT FIFO write command (FROM sdram controller)
			.c0_fifo_read(w_input_fifo_read),		//	Read signal FROM USB
			.ep80_write(reading),						//	Write signal (to perform Acquisition - FROM PC)
			.epA0_read(read_req),						//	Read signal FROM sdram controller
			.ep80fifo_empty(w_input_fifo_empty),	//	input FIFO empty (FROM UUT)
			.ep80fifo_full(w_input_fifo_full),		//	input FIFO full (FROM UUT)
			.epA0fifo_empty(w_output_fifo_empty),	//	output FIFO empty (TO PC)
			.epA0fifo_full(w_output_fifo_full),		//	output FIFO full (TO PC)
			.fault_ofull(fault_ofull),					//	output FIFO has filled up during a transfer to the PC
			.fault_oempty(fault_oempty),				//	output FIFO has emptied during a transfer tot he PC
			.fault_ifull(fault_ifull),					//	input FIFO has filled up during an acquisition
			.fault_iempty(fault_iempty)				//	input FIFO is empty during an acquisition
			);
		
	
		
	
endmodule
